from easyprocess import EasyProcess


def test_call():
    for x in range(1000):
        # test for:
        # OSError exception:[Errno 24] Too many open files
        print("index=", x)
        assert EasyProcess("echo hi").call().return_code == 0


#    def test_start(self):
#        for x in range(1000):
#            print('index=', x)
#            EasyProcess('echo hi').start()



import sys

from easyprocess import EasyProcess


def test():
    # skip these tests for Windows/Mac
    if not sys.platform.startswith("linux"):
        return
    assert (
        EasyProcess([sys.executable, "-m", "easyprocess.examples.ver"])
        .call()
        .return_code
        == 0
    )

    assert (
        EasyProcess([sys.executable, "-m", "easyprocess.examples.log"])
        .call()
        .return_code
        == 0
    )
    assert (
        EasyProcess([sys.executable, "-m", "easyprocess.examples.cmd"])
        .call()
        .return_code
        == 0
    )
    assert (
        EasyProcess([sys.executable, "-m", "easyprocess.examples.hello"])
        .call()
        .return_code
        == 0
    )
    assert (
        EasyProcess([sys.executable, "-m", "easyprocess.examples.timeout"])
        .call()
        .return_code
        == 0
    )



import os
import sys
import urllib.request
from os.path import abspath, dirname
from time import sleep

from easyprocess import EasyProcess

webserver_code = """
from http.server import HTTPServer, CGIHTTPRequestHandler
srv = HTTPServer(server_address=("", 8080), RequestHandlerClass=CGIHTTPRequestHandler)
srv.serve_forever()
"""
os.chdir(dirname(abspath(__file__)))
with EasyProcess([sys.executable, "-c", webserver_code]):
    sleep(2)  # wait for server
    html = urllib.request.urlopen("http://localhost:8080").read().decode("utf-8")
print(html)



import sys
import time

import pytest

from easyprocess import EasyProcess

python = sys.executable


def test_timeout():
    p = EasyProcess("sleep 1").start()
    p.wait(0.2)
    assert p.is_alive()
    p.wait(0.2)
    assert p.is_alive()
    p.wait(2)
    assert not p.is_alive()

    assert EasyProcess("sleep 0.3").call().return_code == 0
    assert EasyProcess("sleep 0.3").call(timeout=0.1).return_code != 0
    assert EasyProcess("sleep 0.3").call(timeout=1).return_code == 0

    assert EasyProcess("sleep 0.3").call().timeout_happened is False
    assert EasyProcess("sleep 0.3").call(timeout=0.1).timeout_happened
    assert EasyProcess("sleep 0.3").call(timeout=1).timeout_happened is False


@pytest.mark.timeout(10)
def test_time_cli1():
    hdr = "import logging;logging.basicConfig(level=logging.DEBUG);from easyprocess import EasyProcess;"
    p = EasyProcess(
        [
            python,
            "-c",
            hdr + "EasyProcess('sleep 15').start()",
        ]
    )
    p.call()
    assert p.return_code == 0


@pytest.mark.timeout(10)
def test_time_cli2():
    hdr = "import logging;logging.basicConfig(level=logging.DEBUG);from easyprocess import EasyProcess;"
    p = EasyProcess(
        [
            python,
            "-c",
            hdr + "EasyProcess('sleep 15').call(timeout=0.5)",
        ]
    )
    p.call()
    assert p.return_code == 0


@pytest.mark.timeout(10)
def test_time2():
    p = EasyProcess("sleep 15").call(timeout=1)
    assert p.is_alive() is False
    assert p.timeout_happened
    assert p.return_code != 0
    assert p.stdout == ""


@pytest.mark.timeout(10)
def test_timeout_out():
    p = EasyProcess(
        [python, "-c", "import time;print( 'start');time.sleep(15);print( 'end')"]
    ).call(timeout=1)
    assert p.is_alive() is False
    assert p.timeout_happened
    assert p.return_code != 0
    assert p.stdout == ""


@pytest.mark.timeout(3)
def test_time3():
    EasyProcess("sleep 15").start()


ignore_term = """
import signal;
import time;
signal.signal(signal.SIGTERM, lambda *args: None);
while True:
    time.sleep(0.5);
"""


# @pytest.mark.timeout(10)
# def test_force_timeout():
#     proc = EasyProcess([python, "-c", ignore_term]).start()
#     # Calling stop() right away actually stops python before it
#     # has a change to actually compile and run the input code,
#     # meaning the signal handlers aren't registered yet. Give it
#     # a moment to setup
#     time.sleep(1)
#     proc.stop(kill_after=1)
#     assert proc.is_alive() is False
#     assert proc.return_code != 0


@pytest.mark.timeout(30)
def test_kill():
    proc = EasyProcess([python, "-c", ignore_term]).start()
    # Calling stop() right away actually stops python before it
    # has a change to actually compile and run the input code,
    # meaning the signal handlers aren't registered yet. Give it
    # a moment to setup
    time.sleep(3)
    proc.stop()
    assert proc.is_alive() is False
    assert proc.return_code != 0


# @pytest.mark.timeout(10)
# def test_force_0_timeout():
#     proc = EasyProcess([python, "-c", ignore_term]).start()
#     time.sleep(1)
#     proc.stop(kill_after=0)
#     assert proc.is_alive() is False
#     assert proc.return_code != 0


@pytest.mark.timeout(10)
def test_force_timeout2():
    proc = EasyProcess([python, "-c", ignore_term]).call(timeout=1)
    assert proc.is_alive() is False
    assert proc.return_code != 0


# @pytest.mark.timeout(10)
# def test_stop_wait():
#     proc = EasyProcess([python, "-c", ignore_term]).start()
#     time.sleep(1)
#     proc.sendstop().wait(timeout=1)
#     # On windows, Popen.terminate actually behaves like kill,
#     # so don't check that our hanging process code is actually hanging.
#     # The end result is still what we want. On other platforms, leave
#     # this assertion to make sure we are correctly testing the ability
#     # to stop a hung process
#     if not sys.platform.startswith("win"):
#         assert proc.is_alive() is True
#     proc.stop(kill_after=1)
#     assert proc.is_alive() is False
#     assert proc.return_code != 0


@pytest.mark.timeout(30)
def test_stop_wait():
    proc = EasyProcess([python, "-c", ignore_term]).start()
    time.sleep(3)
    proc.sendstop().wait(timeout=3)
    assert proc.is_alive() is False
    assert proc.return_code != 0



import pytest

from easyprocess import EasyProcess

# if run with coverage:
#        Fatal Python error: deallocating None


@pytest.mark.timeout(1000)
def test_timeout():  # pragma: no cover
    for x in range(1000):
        print("index=", x)
        assert EasyProcess("sleep 5").call(timeout=0.05).return_code != 0



import json
import sys

from easyprocess import EasyProcess

python = sys.executable


def pass_env(e):
    prog = "import os,json;print(json.dumps(dict(os.environ)))"
    s = EasyProcess([python, "-c", prog], env=e).call().stdout
    return json.loads(s)


def test_env():
    assert len(pass_env(None)) > 0
    e = pass_env(None)
    assert pass_env(e).get("FOO") is None
    e["FOO"] = "2"
    assert pass_env(e).get("FOO") == "2"



from easyprocess import EasyProcess


def test_return_code():
    # process has finished but no stop() or wait() was called
    assert EasyProcess("echo hello").start().sleep(0.5).return_code is None

    # wait()
    assert EasyProcess("echo hello").start().wait().return_code == 0

    # stop() after process has finished
    assert EasyProcess("echo hello").start().sleep(0.5).stop().return_code == 0

    # stop() before process has finished
    assert EasyProcess("sleep 2").start().stop().return_code != 0

    # same as start().wait().stop()
    assert EasyProcess("echo hello").call().return_code == 0


def test_is_alive1():
    # early exit
    p = EasyProcess("echo hello").start().sleep(0.5)

    assert p.return_code is None
    assert p.stdout is None
    assert p.stderr is None

    assert p.is_alive() is False  # is_alive collects ouputs if proc stopped

    assert p.return_code == 0
    assert p.stdout == "hello"
    assert p.stderr == ""


def test_is_alive2():
    # no exit
    p = EasyProcess("sleep 10").start()

    assert p.return_code is None
    assert p.stdout is None
    assert p.stderr is None

    assert p.is_alive()  # is_alive collects ouputs if proc stopped

    assert p.return_code is None
    assert p.stdout is None
    assert p.stderr is None



import logging
import shlex
from typing import List

log = logging.getLogger(__name__)


class EasyProcessUnicodeError(Exception):
    pass


def split_command(cmd, posix=None) -> List[str]:
    """
     - cmd is string list -> nothing to do
     - cmd is string -> split it using shlex

    :param cmd: string ('ls -l') or list of strings (['ls','-l'])
    :rtype: string list
    """
    if not isinstance(cmd, str):
        # cmd is string list
        pass
    else:
        if posix is None:
            posix = True
        cmd = shlex.split(cmd, posix=posix)
    return cmd


# def uniencode(s):
#     return s


def unidecode(s):
    s = s.decode("utf-8", "ignore")
    return s



import pytest

from easyprocess import EasyProcess, EasyProcessError


def test_is_started():
    assert EasyProcess("ls -la").is_started is False
    assert EasyProcess("ls -la").start().is_started
    assert EasyProcess("ls -la").call().is_started
    assert EasyProcess("ls -la").start().wait().is_started
    assert EasyProcess("ls -la").start().stop().is_started


def test_raise():
    with pytest.raises(EasyProcessError):
        EasyProcess("ls -la").start().start()
    with pytest.raises(EasyProcessError):
        EasyProcess("ls -la").stop()
    with pytest.raises(EasyProcessError):
        EasyProcess("ls -la").sendstop()
    # .assertRaises(EasyProcessError, lambda : EasyProcess('ls
    # -la').start().stop().stop())
    with pytest.raises(EasyProcessError):
        EasyProcess("ls -la").start().wrap(lambda: None)()
    EasyProcess("ls -la").wrap(lambda: None)()



import sys

from easyprocess import EasyProcess
from easyprocess.unicodeutil import split_command

OMEGA = "\u03A9"

python = sys.executable


def platform_is_win():
    return sys.platform == "win32"


def py_minor():
    return sys.version_info[1]


def test_str():
    assert EasyProcess("ls -la").call().return_code == 0


def test_ls():
    assert EasyProcess(["ls", "-la"]).call().return_code == 0


def test_parse():
    assert EasyProcess("ls -la").cmd == ["ls", "-la"]
    assert EasyProcess('ls "abc"').cmd == ["ls", "abc"]
    assert EasyProcess('ls "ab c"').cmd == ["ls", "ab c"]


def test_split():
    # list -> list
    assert split_command([str("x"), str("y")]) == ["x", "y"]
    assert split_command([str("x"), "y"]) == ["x", "y"]
    assert split_command([str("x"), OMEGA]) == ["x", OMEGA]

    # str -> list
    assert split_command(str("x y")) == ["x", "y"]
    assert split_command("x y") == ["x", "y"]
    assert split_command("x " + OMEGA) == ["x", OMEGA]

    # split windows paths #12
    assert split_command("c:\\temp\\a.exe someArg", posix=False) == [
        "c:\\temp\\a.exe",
        "someArg",
    ]


def test_echo():
    assert EasyProcess("echo hi").call().stdout == "hi"
    if not platform_is_win():
        assert EasyProcess("echo " + OMEGA).call().stdout == OMEGA
        assert EasyProcess(["echo", OMEGA]).call().stdout == OMEGA


def test_argv():
    assert (
        EasyProcess([python, "-c", r"import sys;assert sys.argv[1]=='123'", "123"])
        .call()
        .return_code
        == 0
    )
    assert (
        EasyProcess([python, "-c", r"import sys;assert sys.argv[1]=='\u03a9'", OMEGA])
        .call()
        .return_code
        == 0
    )


if py_minor() > 6:  # sys.stdout.reconfigure from py3.7

    def test_py_print():
        assert (
            EasyProcess(
                [
                    python,
                    "-c",
                    r"import sys;sys.stdout.reconfigure(encoding='utf-8');print('\u03a9')",
                ]
            )
            .call()
            .stdout
            == OMEGA
        )

        assert (
            EasyProcess(
                [
                    python,
                    "-c",
                    r"import sys;sys.stdout.reconfigure(encoding='utf-8');print(sys.argv[1])",
                    OMEGA,
                ]
            )
            .call()
            .stdout
            == OMEGA
        )


def test_py_stdout_write():
    assert (
        EasyProcess(
            [
                python,
                "-c",
                r"import sys;sys.stdout.buffer.write('\u03a9'.encode('utf-8'))",
            ]
        )
        .call()
        .stdout
        == OMEGA
    )


def test_invalid_stdout():
    """invalid utf-8 byte in stdout."""
    # https://en.wikipedia.org/wiki/UTF-8#Codepage_layout

    # 0x92  continuation byte
    cmd = [python, "-c", "import sys;sys.stdout.buffer.write(b'\\x92')"]
    p = EasyProcess(cmd).call()
    assert p.return_code == 0
    assert p.stdout == ""

    # 0xFF must never appear in a valid UTF-8 sequence
    cmd = [python, "-c", "import sys;sys.stdout.buffer.write(b'\\xFF')"]
    p = EasyProcess(cmd).call()
    assert p.return_code == 0
    assert p.stdout == ""



import sys

from easyprocess import EasyProcess

python = sys.executable

print("-- Run program, wait for it to complete, get stdout:")
s = EasyProcess([python, "-c", "print(3)"]).call().stdout
print(s)

print("-- Run program, wait for it to complete, get stderr:")
s = EasyProcess([python, "-c", "import sys;sys.stderr.write('4\\n')"]).call().stderr
print(s)

print("-- Run program, wait for it to complete, get return code:")
s = EasyProcess([python, "--version"]).call().return_code
print(s)

print("-- Run program, wait 1.5 second, stop it, get stdout:")
prog = """
import time
for i in range(10):
    print(i, flush=True)
    time.sleep(1)
"""
s = EasyProcess([python, "-c", prog]).start().sleep(1.5).stop().stdout
print(s)



import os
import sys
import threading
from time import sleep

import pytest
from pyvirtualdisplay.display import Display

from easyprocess import EasyProcess

python = sys.executable
# requirement:  apt install imagemagick

# deadlock
# popen.communicate() hangs
# no deadlock with temp_files

PROG = """
from PIL import Image
Image.new("RGB",(99, 99)).show()
"""

EASYPROCESS_USE_TEMP_FILES = os.environ.get("EASYPROCESS_USE_TEMP_FILES")


def test_dummy():
    pass


# skip these tests for Windows/Mac
# and when 'use_temp_files' is forced by env variable
if sys.platform.startswith("linux") and not EASYPROCESS_USE_TEMP_FILES:

    def test_has_imagemagick():
        assert EasyProcess(["display", "-version"]).call().return_code == 0

    @pytest.mark.timeout(120)
    def test_deadlock_temp_files():
        with Display():
            p = EasyProcess(
                [
                    python,
                    "-c",
                    PROG,
                ],
                use_temp_files=True,
            )
            p.start()
            sleep(2)
            # hangs with pipes
            p.stop()

    @pytest.mark.timeout(120)
    def test_deadlock_pipe():
        with Display():
            p = EasyProcess(
                [
                    python,
                    "-c",
                    PROG,
                ],
                use_temp_files=False,
            )
            p.start()
            sleep(2)

            def start():
                # hangs with pipes
                p.stop()

            thread = threading.Thread(target=start)
            thread.start()

            sleep(6)
            assert thread.is_alive()

        thread.join()



from __future__ import with_statement

import sys
import time

import pytest

from easyprocess import EasyProcess

python = sys.executable


def test_call():
    assert EasyProcess("ls -la").call().return_code == 0
    assert EasyProcess(["ls", "-la"]).call().return_code == 0


def test_start():
    p = EasyProcess("ls -la").start()
    time.sleep(0.2)
    assert p.stop().return_code == 0


def test_start2():
    p = EasyProcess("echo hi").start()
    time.sleep(0.2)
    # no wait() -> no results
    assert p.return_code is None
    assert p.stdout is None


@pytest.mark.timeout(1)
def test_start3():
    p = EasyProcess("sleep 10").start()
    assert p.return_code is None


def test_alive():
    assert EasyProcess("ping 127.0.0.1 -c 2").is_alive() is False
    assert EasyProcess("ping 127.0.0.1 -c 2").start().is_alive()
    assert EasyProcess("ping 127.0.0.1 -c 2").start().stop().is_alive() is False
    assert EasyProcess("ping 127.0.0.1 -c 2").call().is_alive() is False


def test_std():
    assert EasyProcess("echo hello").call().stdout == "hello"
    assert EasyProcess([python, "-c", "print(42)"]).call().stdout == "42"


def test_wait():
    assert EasyProcess("echo hello").wait().return_code is None
    assert EasyProcess("echo hello").wait().stdout is None

    assert EasyProcess("echo hello").start().wait().return_code == 0
    assert EasyProcess("echo hello").start().wait().stdout == "hello"


#     def test_xephyr():
#         EasyProcess('Xephyr -help').check(return_code=1)


def test_wrap():
    def f():
        return EasyProcess("echo hi").call().stdout

    assert EasyProcess("ping 127.0.0.1").wrap(f)() == "hi"


def test_with():
    with EasyProcess("ping 127.0.0.1") as x:
        assert x.is_alive()
    assert x.return_code != 0
    assert not x.is_alive()


def test_parse():
    assert EasyProcess("ls -la").cmd == ["ls", "-la"]
    assert EasyProcess('ls "abc"').cmd == ["ls", "abc"]
    assert EasyProcess('ls "ab c"').cmd == ["ls", "ab c"]


def test_stop():
    p = EasyProcess("ls -la").start()
    time.sleep(0.2)
    assert p.stop().return_code == 0
    assert p.stop().return_code == 0
    assert p.stop().return_code == 0



import glob
import logging
import os

from entrypoint2 import entrypoint

from easyprocess import EasyProcess

commands = [
    "python3 -m easyprocess.examples.hello",
    "python3 -m easyprocess.examples.cmd",
    "python3 -m easyprocess.examples.timeout",
    "python3 -m easyprocess.examples.with",
]


def empty_dir(dir):
    files = glob.glob(os.path.join(dir, "*"))
    for f in files:
        os.remove(f)


@entrypoint
def main():
    gendir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "gen")
    logging.info("gendir: %s", gendir)
    os.makedirs(gendir, exist_ok=True)
    empty_dir(gendir)
    pls = []
    try:
        os.chdir("gen")
        for cmd in commands:
            logging.info("cmd: %s", cmd)
            fname_base = cmd.replace(" ", "_")
            fname = fname_base + ".txt"
            logging.info("cmd: %s", cmd)
            print("file name: %s" % fname)
            with open(fname, "w") as f:
                f.write("$ " + cmd + "\n")
                p = EasyProcess(cmd).call()
                f.write(p.stdout)
                f.write(p.stderr)
                pls += [p]
    finally:
        os.chdir("..")
        for p in pls:
            p.stop()
    embedme = EasyProcess(["npx", "embedme", "../README.md"])
    embedme.call()
    print(embedme.stdout)
    assert embedme.return_code == 0
    assert "but file does not exist" not in embedme.stdout



#!/usr/bin/env python3
import pathlib
import shutil

[shutil.rmtree(p) for p in pathlib.Path(".").glob(".tox")]
[shutil.rmtree(p) for p in pathlib.Path(".").glob("dist")]
[shutil.rmtree(p) for p in pathlib.Path(".").glob("*.egg-info")]
[shutil.rmtree(p) for p in pathlib.Path(".").glob("build")]
[p.unlink() for p in pathlib.Path(".").rglob("*.py[co]")]
[p.rmdir() for p in pathlib.Path(".").rglob("__pycache__")]
[p.unlink() for p in pathlib.Path(".").rglob("*.log")]
